/*
 * P4.Net *
Copyright (c) 2006 Shawn Hladky

Permission is hereby granted, free of charge, to any person obtaining a copy of this software 
and associated documentation files (the "Software"), to deal in the Software without 
restriction, including without limitation the rights to use, copy, modify, merge, publish, 
distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the 
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or 
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
 */


using System;
using System.Text;
using System.Collections;
using System.Collections.Specialized;

namespace P4API
{
    /// <summary>
    /// P4Record is a dictionary-like object that provides a means of interpreting Perforce results.
    /// </summary>
    /// <remarks>
    /// The Perforce api provides "parsed" output in the form of key-value pairs.  
    /// Keys are always strings (case sensitive).  From the raw api, the values come in 3 flavors:
    /// <list>
    ///     <li>Simple strings.</li>
    ///     <li>List of strings.</li>
    ///     <li>Sparse multi-demensional list of strings.</li>    
    /// </list>
    /// P4.Net does not yet handle the third type (which only comes from certain output of p4 filelog).
    /// <para>To access simple string values by key, use the Fields property (or the default indexer).</para>
    /// <para>To access lists of strings by key, use the ArrayFields property.</para>
    /// </remarks>
    public class P4Record
    {
        internal FieldDictionary _Fields = null;
        internal ArrayFieldDictionary _ArrayFields = null;

        internal P4Record(Hashtable sd)
        {
            _Fields = new FieldDictionary();
            _ArrayFields = new ArrayFieldDictionary();

            // first copy the Hashtable
            Hashtable ht = (Hashtable)sd.Clone();
            ArrayList keys = new ArrayList(ht.Keys);
            foreach (string s in keys)
            {
                // the key may have been removed if it was an array,
                // so check here for that
                if (ht.ContainsKey(s))
                {
                    string keyBaseName = s;

                    int lastNumericCharacter = s.Length;

                    // look to see if the variable is an array
                    // we define an array as:
                    // there must exist a key that is all alpha followed by exactly one '0'
                    // if that key exists, all other keys with that base alpha name followed by a number
                    // will be an element in the array indexed by said number.
                    while (s[lastNumericCharacter - 1] >= '0' && s[lastNumericCharacter - 1]<= '9')
                    {
                        lastNumericCharacter--;
                    }
                    if (lastNumericCharacter != s.Length)
                    {
                        keyBaseName = s.Substring(0, lastNumericCharacter);

                        if (ht.ContainsKey(string.Format("{0}0", keyBaseName)))
                        {
                            int i = 0;
                            ArrayList list = new ArrayList();
                            //spin the array 
                            string indexKey = string.Format("{0}0", keyBaseName);
                            do
                            {
                                list.Add((string)ht[indexKey]);
                                // remove it from the hashtable so we don't hit it again
                                ht.Remove(indexKey);
                                i++;
                                indexKey = string.Format("{0}{1}", keyBaseName, i);
                            }
                            while (ht.ContainsKey(indexKey));

                            string[] sList = (string[])list.ToArray(typeof(string));
                            _ArrayFields.Add(keyBaseName, sList);
                        }
                    }
                    else
                    {
                        _Fields.Add(s, (string)ht[s]);
                    }                    
                }
            }
        }

        /// <summary>
        /// Gets the FieldDictionary returned from the Perforce command.
        /// </summary>
        public FieldDictionary Fields
        {
            get
            {
                return _Fields;
            }
        }

        /// <summary>
        /// Gets the ArrayFieldDictionary returned from the Perforce command.
        /// </summary>
        public ArrayFieldDictionary ArrayFields
        {
            get
            {
                return _ArrayFields;
            }
        }

        /// <summary>
        /// Returns the value of of the Field by key.  This is the same as Fields[key].
        /// </summary>
        /// <param name="key">The key for </param>
        /// <returns>String value for the associated key.</returns>
        public string this[string key]
        {
            get
            {
                return _Fields[key];
            }
            set
            {
                _Fields[key] = value;
            }
        }

        internal Hashtable AllFieldDictionary
        {
            get 
            {
                Hashtable ret = new Hashtable();

                // Add the scalar fields
                foreach (string key in _Fields.Keys)
                {
                    ret.Add(key, _Fields[key]);
                }

                // Add the array fields
                foreach (string key in _ArrayFields.Keys)
                {
                    for (int i = 0; i < _ArrayFields[key].Length; i++)
                    {
                        ret.Add(string.Format("{0}{1}", key, i), _ArrayFields[key][i]);
                    }
                    
                }
                return ret;
            }

        }


    }
}
